==> web2ldap-0.16.38.tar.gz: web2ldap-0.16.38/pylib/ldaputil/schema.py <==
"""
ldaputil.schema: More functionality for ldap.schema

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: schema.py,v 1.4 2008/07/11 15:54:45 michael Exp $
"""

import ldap.schema,ldap.schema.subentry

from ldap.schema.models import AttributeType

class SubSchema(ldap.schema.subentry.SubSchema):

  def __init__(self,sub_schema_sub_entry):
    ldap.schema.subentry.SubSchema.__init__(self,sub_schema_sub_entry)
    self.no_user_mod_attr_oids = self.determine_no_user_mod_attrs()

  def determine_no_user_mod_attrs(self):
    result = {}.fromkeys([
      a.oid
      for a in self.sed[ldap.schema.AttributeType].values()
      if a.no_user_mod
    ])
    return result # determine_no_user_mod_attrs()


class NameForm(ldap.schema.models.SchemaElement):
  """
  NameFormDescription = LPAREN WSP
      numericoid                 ; object identifer
      [ SP "NAME" SP qdescrs ]   ; short names
      [ SP "DESC" SP qdstring ]  ; description
      [ SP "OBSOLETE" ]          ; not active
      SP "OC" SP oid             ; structural object class
      SP "MUST" SP oids          ; attribute types
      [ SP "MAY" SP oids ]       ; attribute types
      extensions WSP RPAREN      ; extensions
  """
  schema_attribute = 'nameForms'
  token_defaults = {
    'NAME':(()),
    'DESC':(None,),
    'OBSOLETE':None,
    'OC':(None,),
    'MUST':(()),
    'MAY':(()),
  }

  def _set_attrs(self,l,d):
    self.names = d['NAME']
    self.desc = d['DESC'][0]
    self.obsolete = d['OBSOLETE']!=None
    self.oc = d['OC'][0]
    self.must = d['MUST']
    self.may = d['MAY']
    assert type(self.names)==TupleType
    assert self.desc is None or type(self.desc)==StringType
    assert type(self.obsolete)==BooleanType and (self.obsolete==0 or self.obsolete==1)
    assert type(self.oc)==StringType
    assert type(self.must)==TupleType
    assert type(self.may)==TupleType
    return

  def __str__(self):
    result = [str(self.oid)]
    result.append(self.key_list('NAME',self.names,quoted=1))
    result.append(self.key_attr('DESC',self.desc,quoted=1))
    result.append({0:'',1:' OBSOLETE'}[self.obsolete])
    result.append(self.key_attr('OC',self.oc))
    result.append(self.key_list('MUST',self.must,sep=' $ '))
    result.append(self.key_list('MAY',self.may,sep=' $ '))
    return '( %s )' % ''.join(result)


#------------------------------------------------
# Work arounds for older versions of python-ldap
#------------------------------------------------

# Override possibly buggy version in python-ldap
ldap.schema.Nameform = NameForm

# Do not treat the BitString and OctetString syntaxes as not human-readable
try:
  del ldap.schema.NOT_HUMAN_READABLE_LDAP_SYNTAXES['1.3.6.1.4.1.1466.115.121.1.6']
  del ldap.schema.NOT_HUMAN_READABLE_LDAP_SYNTAXES['1.3.6.1.4.1.1466.115.121.1.40']
except KeyError:
  pass

==> web2ldap-0.16.40.tar.gz: web2ldap-0.16.40/pylib/ldaputil/schema.py <==
"""
ldaputil.schema: More functionality for ldap.schema

(c) by Michael Stroeder <michael@stroeder.com>

This module is distributed under the terms of the
GPL (GNU GENERAL PUBLIC LICENSE) Version 2
(see http://www.gnu.org/copyleft/gpl.html)

$Id: schema.py,v 1.5 2008/07/24 10:03:34 michael Exp $
"""

import ldap.schema,ldap.schema.subentry,msbase

from ldap.schema.models import AttributeType


class SubSchema(ldap.schema.subentry.SubSchema):

  def __init__(self,sub_schema_sub_entry):
    ldap.schema.subentry.SubSchema.__init__(self,sub_schema_sub_entry)
    self.no_user_mod_attr_oids = self.determine_no_user_mod_attrs()

  def determine_no_user_mod_attrs(self):
    result = {}.fromkeys([
      a.oid
      for a in self.sed[ldap.schema.AttributeType].values()
      if a.no_user_mod
    ])
    return result # determine_no_user_mod_attrs()


class NameForm(ldap.schema.models.SchemaElement):
  """
  NameFormDescription = LPAREN WSP
      numericoid                 ; object identifer
      [ SP "NAME" SP qdescrs ]   ; short names
      [ SP "DESC" SP qdstring ]  ; description
      [ SP "OBSOLETE" ]          ; not active
      SP "OC" SP oid             ; structural object class
      SP "MUST" SP oids          ; attribute types
      [ SP "MAY" SP oids ]       ; attribute types
      extensions WSP RPAREN      ; extensions
  """
  schema_attribute = 'nameForms'
  token_defaults = {
    'NAME':(()),
    'DESC':(None,),
    'OBSOLETE':None,
    'OC':(None,),
    'MUST':(()),
    'MAY':(()),
  }

  def _set_attrs(self,l,d):
    self.names = d['NAME']
    self.desc = d['DESC'][0]
    self.obsolete = d['OBSOLETE']!=None
    self.oc = d['OC'][0]
    self.must = d['MUST']
    self.may = d['MAY']
    assert type(self.names)==TupleType
    assert self.desc is None or type(self.desc)==StringType
    assert type(self.obsolete)==BooleanType and (self.obsolete==0 or self.obsolete==1)
    assert type(self.oc)==StringType
    assert type(self.must)==TupleType
    assert type(self.may)==TupleType
    return

  def __str__(self):
    result = [str(self.oid)]
    result.append(self.key_list('NAME',self.names,quoted=1))
    result.append(self.key_attr('DESC',self.desc,quoted=1))
    result.append({0:'',1:' OBSOLETE'}[self.obsolete])
    result.append(self.key_attr('OC',self.oc))
    result.append(self.key_list('MUST',self.must,sep=' $ '))
    result.append(self.key_list('MAY',self.may,sep=' $ '))
    return '( %s )' % ''.join(result)


class Entry(ldap.schema.models.Entry):
  """
  Base class with some additional basic methods
  """

  def get_associated_name_form_objs(self):
    structural_object_class_oid = self._s.get_structural_oc(self['objectClass'])
    structural_object_class_obj = self._s.get_obj(ldap.schema.models.ObjectClass,structural_object_class_oid)
    if structural_object_class_obj:
      structural_object_class_names = [
        oc_name.lower()
	for oc_name in structural_object_class_obj.names or ()
      ]
    else:
      structural_object_class_names = ()
    result = []
    for name_form_oid,name_form_obj in self._s.sed[ldap.schema.models.NameForm].items():
      if not name_form_obj.obsolete and (
           name_form_obj.oc==structural_object_class_oid or \
           name_form_obj.oc.lower() in structural_object_class_names
      ):
        result.append(name_form_obj)
    return result # get_associated_name_form_objs()

  def get_rdn_templates(self):
    rdn_variants = []
    for name_form_obj in self.get_associated_name_form_objs():
      rdn_variants.append(name_form_obj.must)
      for i in msbase.combinations(name_form_obj.may):
	rdn_variants.append(name_form_obj.must+i)
    # Finally convert the tuple RDN combinations to strings
    return [
      '+'.join([
	'%s=' % (attr_type)
	for attr_type in attr_types
      ])
      for attr_types in rdn_variants
    ] # get_rdn_templates()


#------------------------------------------------
# Work arounds for older versions of python-ldap
#------------------------------------------------

# Override possibly buggy version in python-ldap
ldap.schema.Nameform = NameForm

# Do not treat the BitString and OctetString syntaxes as not human-readable
try:
  del ldap.schema.NOT_HUMAN_READABLE_LDAP_SYNTAXES['1.3.6.1.4.1.1466.115.121.1.6']
  del ldap.schema.NOT_HUMAN_READABLE_LDAP_SYNTAXES['1.3.6.1.4.1.1466.115.121.1.40']
except KeyError:
  pass
